// FILin fileechoprocessor
// Copyright (c) 1995-99 Ilya Bogdanov
// See licence.txt for details

/* ----- DIRFILE.C -----
 * Functions for file and directory processing */

#include <direct.h>
#include <io.h>
#include <fcntl.h>
#include <sys\stat.h>
#include "general.h"

char *dirfull(
    char *subdir )
{
    char *buff;
    if( LASTCHAR( subdir ) == '\\' )                // for OS/2 _fullpath
        LASTCHAR( subdir ) = 0;
    strupr( strcpy( subdir, buff = _fullpath( NULL, subdir, 0 ) ) );
    free( buff );
    return strcat( subdir, "\\" );
}

/* --- filefull ---
 * fullname = subdir + filename */

char *filefull (char *fullname, char *subdir, char *filename)
{
//#if defined(DOS)
//  return strcat(strcpy(fullname,subdir),filename);
//#else
  return strncat(strcpy(fullname,subdir),filename,PATHLEN-strlen(subdir)-1);
//#endif
}

/* --- copyfile ---
 * copies infile to directory subdir by 32K blocks
 * if file exists old attributes are saved */

void copyfile (char *infile, char *subdir)
{
  char *temp, indir[DIRLEN], tofile[PATHLEN];
  int sfile,dfile,readed;
  static void *pktbuf;
  unsigned short date,time;     // _dos_getftime(), _dos_setftime()

  if ((temp = strrchr(infile,'\\')) != NULL)  // load indir if there's dir
  {                                           // in infile
    strncpy(indir,infile,temp-infile+1);
    indir[temp-infile+1] = 0;
  }
  else
     *indir = 0;                              // else indir = 0 (for dirfull)
  if (!strcmp(dirfull(indir),dirfull(subdir))) return; // copy to itself
  if (temp) filefull(tofile,subdir,temp+1);   // subdir + name of file
  else filefull(tofile,subdir,infile);        // subdir + infile
  if (!pktbuf)                                // the first call to copyfile
    if ((pktbuf = malloc(0x8000)) == NULL) errexit(3,"pktbuf");

  if ((sfile = open(infile,O_RDONLY|O_BINARY)) == -1)
     errexit(1,infile);
  if ((dfile = open(tofile,O_WRONLY|O_BINARY|O_CREAT|O_TRUNC,S_IWRITE)) == -1)
     errexit(1,tofile);    // truncate if exists, create for write if not
  readed = read(sfile,pktbuf,0x8000);
  while (readed)
  {
    if (write(dfile,pktbuf,readed) < readed)
       errexit(0,"Bad writing. Disk full?");
    readed = read(sfile,pktbuf,0x8000);
  }
  _dos_getftime(sfile,&date,&time);     // dest file should have the same date
  _dos_setftime(dfile,date,time);
  close(dfile);
  close(sfile);
}

/* --- movefile ---
 * moves infile to directory subdir
 * renames if possible
 * calls copyfile() if not */

void movefile (char *infile, char *subdir)
{
  char *temp, tofile[PATHLEN];

  dirfull(subdir);
  temp = strrchr(infile,'\\');
  if (temp) filefull(tofile,subdir,temp+1);   // subdir + name of file
  else filefull(tofile,subdir,infile);        // subdir + infile

  if (rename(infile,tofile) == -1)           // if rename to itself -- OK?
  {
    copyfile(infile,subdir);
    unlink(infile);
  }
}

int makexist        // 1 if subdir exists or has been created; 0 if error
    (char *subdir)
{
    char *p, tempdir[DIRLEN];
    strcat( strcpy( tempdir, dirfull( subdir ) ), "." );   // "c:\home\dir\."
    if (access(tempdir,F_OK) == 0) return 1;    // subdir already exists
    do
    {
        p = strchr(tempdir,0);                  
        if (*(p-3) == ':') return 0;            // if "c:\." doesn't exist
        *(p-2) = 0;                             // "c:\home\dir"
        p = strrchr(tempdir,'\\');
        *(p+1) = '.';
        *(p+2) = 0;                             // "c:\home\."
    }
    while (access(tempdir,F_OK) != 0);          // till existing parent dir
    strcpy(tempdir,subdir);                     // "c:\home\dir\"
    while (p = strchr(p+1,'\\'))                 
    {
        *p = 0;                                 // "c:\home\dir"
        mkdir(tempdir);
        *p = '\\';                              // "c:\home\dir\"
    }
    strcat(tempdir,".");                        // "c:\home\dir\."
    return (access(tempdir,F_OK) == 0);
}

int fileage             // how old is the file (in days); -1 if error
    (char *filename)
{
    struct stat buf;
    if (stat(filename,&buf) == -1) return -1;
    return difftime(time(NULL),buf.st_mtime)/(24*3600);
}
