// FILin fileechoprocessor
// Copyright (c) 1995-99 Ilya Bogdanov
// See licence.txt for details

#include <stdio.h>
#include <sys\stat.h>
#include <string.h>
#include <dos.h>
#include <time.h>
#include <stdlib.h>
#include <ctype.h>
#include "fidomsg.h"

char *msgtime (void)
{
  static char totime[20];
  time_t t;
  char timestr[26];
  int k;

  time(&t);
  strcpy(timestr,ctime(&t));
  totime[0] = timestr[8];
  totime[1] = timestr[9];
  totime[2] = ' ';
  totime[3] = timestr[4];
  totime[4] = timestr[5];
  totime[5] = timestr[6];
  totime[6] = ' ';
  totime[7] = timestr[22];
  totime[8] = timestr[23];
  totime[9] = ' ';
  totime[10] = ' ';
  for( k = 0; k < 8; ++k ) totime[11+k] = timestr[11+k];
  return totime;
}

void hdr2msg(
    struct sMsghdr *msghdr,
    struct sMsg *msg )
{
    msg->to.net = msghdr->tonet;
    msg->to.node = msghdr->tonode;
    msg->from.net = msghdr->fromnet;
    msg->from.node = msghdr->fromnode;
    strcpy( msg->fromname, msghdr->fromname );
    strcpy( msg->toname, msghdr->toname );
    strcpy( msg->subj,msghdr->subj );
    msg->isatt = msghdr->flags&0x0010;
    msg->isrcv = msghdr->flags&0x0004;
}

void body2msg(
    char *body,
    struct sMsg *msg )
{
    char *endtext, *p = body;
    int textsize;
    while (*p == 1)         // kludges
    {
        ++p;
        if( !strncmp( p, "FMPT ", 5 ) ) msg->from.point = atoi( p + 5 );
        else
        if( !strncmp( p, "TOPT ", 5 ) ) msg->to.point = atoi( p + 5 );
        else
        if( !strncmp( p, "INTL ", 5 ) )
        {
            p += 5;
            while( isspace( *p ) ) p++;
            msg->to.zone = atoi( p );
            while( !isspace( *p ) ) p++;
            while( isspace( *p ) ) p++;
            msg->from.zone = atoi( p );
        }
        if( ( p = strchr( p, 0x0D ) ) == NULL) break;
        p++;
        if( *p == 0x0A ) p++;    // 0D0A
    }
    if( ( endtext = strstr( p, "---" ) ) == NULL )      // tearline
        if( ( endtext = strchr( p, 1 ) ) == NULL )      // Via
            endtext = strchr( p, 0 );                   // null terminated text (should be)
    msg->text = ( char * ) malloc( textsize = endtext - p + 1 );    // null terminated
    strncpy( msg->text, p, textsize - 1 );
    *( msg->text + textsize - 1 ) = 0;
}

int readmsg(
    char *msgpath,
    struct sMsg *msg,
    unsigned short defaultzone )
{
    FILE *fMsg;
    struct sMsghdr msghdr;
    struct stat buf;
    int bodysize;
    char *body;
    if( stat( msgpath, &buf ) == -1 ) return -1;
    if( ( fMsg = fopen( msgpath, "rb" ) ) == NULL ) return -1;
    if( fread( &msghdr, sizeof( struct sMsghdr ), 1, fMsg ) == 0 ) return -1;
    if( ( bodysize = buf.st_size - sizeof( struct sMsghdr ) ) != 0 )
    {
        if( ( body = ( char * ) malloc( bodysize ) ) == NULL ) return -1;
        if( fread( body, bodysize, 1, fMsg ) == 0 ) return -1;
    }
    fclose( fMsg );
    msg->from.zone = msg->to.zone = defaultzone;
    msg->from.point = msg->to.point = 0;
    hdr2msg( &msghdr, msg );
    body2msg( body, msg );
    free( body );
    return 0;
}

int msgtext(
    struct sMsg *msg,
    char *path )
{
    FILE *fp;
    long size;
    if( ( fp = fopen( path, "rb" ) ) == NULL ) return -1;
    fseek( fp, 0L, SEEK_END );
    msg->text = ( char * ) malloc( ( size = ftell( fp ) ) + 1 );    // + \0
    fseek( fp, 0L, SEEK_SET );
    fread( msg->text, size, 1, fp );
    msg->text[ size ] = 0;
    fclose( fp );
    return 0;
}

void msg2hdr(
        struct sMsg     *msg,
    struct sMsghdr      *msghdr )
{
    memset( msghdr, 0, sizeof( struct sMsghdr ) );
    strcpy( msghdr->fromname, msg->fromname );
        strcpy( msghdr->toname, msg->toname );
        strcpy( msghdr->subj, msg->subj );
    strcpy( msghdr->datetime, msgtime() );
        msghdr->fromnet = msg->from.net;
        msghdr->fromnode = msg->from.node;
        msghdr->tonet = msg->to.net;
        msghdr->tonode = msg->to.node;
    msghdr->flags = 0x0101;                     // pvt loc
    if( msg->isatt ) msghdr->flags |= 0x0010;
    if( msg->iskil ) msghdr->flags |= 0x0080;
    if( msg->ishld ) msghdr->flags |= 0x0200;
}

void msg2kldg(
        struct sMsg     *msg,
        char            *msgkldg )
{
        sprintf( msgkldg, "\1INTL %u:%u/%u %u:%u/%u\r", msg->to.zone, msg->to.net,
        msg->to.node, msg->from.zone, msg->from.net, msg->from.node );
        if( msg->from.point )
        sprintf( msgkldg, "%s\1FMPT %u\r", msgkldg,     msg->from.point );
        if( msg->to.point )
        sprintf( msgkldg, "%s\1TOPT %u\r", msgkldg, msg->to.point );
    if( msg->isdir || msg->iskfs )
        sprintf( msgkldg, "%s\1FLAGS%s%s\r", msgkldg,
                ( msg->isdir ) ? " DIR" : "", ( msg->iskfs ) ? " KFS" : "" );
        sprintf( msgkldg, "%s\1MSGID: %u:%u/%u.%u %02x%03x%03x\r", msgkldg,
        msg->from.zone, msg->from.net, msg->from.node, msg->from.point,
        rand() % 0x100, rand() % 0x1000, rand() % 0x1000 );
}

int writemsg(
    char *msgdir,
    struct sMsg *msg )
{
    struct sMsghdr msghdr;
    char msgpath[_MAX_PATH], msgkldg[200];       // 5 * 40 ( INTL )
    unsigned done, n = 0;
    struct find_t f;
    FILE *fMsg;
    msg2hdr( msg, &msghdr );
    msg2kldg( msg, msgkldg );
    sprintf( msgpath, "%s%s", msgdir, "*.msg" );
    done = _dos_findfirst( msgpath, _A_NORMAL, &f );       
    while( !done )
    {                                    
        if( n < atoi( f.name ) ) n = atoi( f.name );
        done = _dos_findnext( &f );                        
    }
    _dos_findclose( &f );
    sprintf( msgpath, "%s%d%s", msgdir, ++n, ".msg" );
    if( ( fMsg = fopen( msgpath, "wb" ) ) == NULL ) return -1;
    if( fwrite( &msghdr, sizeof( struct sMsghdr ), 1, fMsg ) < 1
        || fwrite( msgkldg, strlen( msgkldg ), 1, fMsg ) < 1
        || fwrite( msg->text, strlen( msg->text ) + 1, 1, fMsg ) < 1 )
            return -1;
    fclose( fMsg );
    return 0;
}
